searchData={"items":[{"type":"module","title":"RabbitMQPoolEx","doc":"`RabbitMQPoolEx` is an Elixir library that provides a robust and efficient connection pooling mechanism for RabbitMQ.\nIt leverages `poolboy` to manage a pool of connections, ensuring high performance and reliability in message-driven\napplications.","ref":"RabbitMQPoolEx.html"},{"type":"module","title":"Features - RabbitMQPoolEx","doc":"- **Connection Pooling**: Efficiently manage multiple RabbitMQ connections to handle high-throughput messaging.\n- **Channel Management**: Simplify the process of acquiring and releasing channels from the connection pool.\n- **Fault Tolerance**: Automatically handle connection drops and retries, ensuring minimal disruption to message\n  processing.\n- **Channel Reuse**: Optionally reuse channels within a pool to optimize resource management.\n- **Configurable Pooling Strategy**: Customize the size, overflow, and behavior of connection pools.\n- **Pool Metrics**: Built-in telemetry metrics for monitoring pools.","ref":"RabbitMQPoolEx.html#module-features"},{"type":"module","title":"Installation - RabbitMQPoolEx","doc":"To integrate `RabbitMQPoolEx` into your project, add the following to your `mix.exs` dependencies:\n\n```elixir\ndefp deps do\n  [\n    {:rabbitmq_pool_ex, \"~> 1.0.0\"}\n  ]\nend\n```\n\nThen, fetch and install the dependencies by running:\n\n```sh\nmix deps.get\n```","ref":"RabbitMQPoolEx.html#module-installation"},{"type":"module","title":"Configuration - RabbitMQPoolEx","doc":"`RabbitMQPoolEx` requires defining connection pools and RabbitMQ settings. The configuration consists of:\n\n- `:rabbitmq_config` (keyword list) – General RabbitMQ connection parameters.\n- `:connection_pools` (list) – A list of poolboy configurations, where each pool represents a connection to RabbitMQ.\n\nEach pool configuration should include:\n\n- `:name` (tuple) – A two element tuple containing the process registration scope and an unique name for the pool (e.g., {:local, :default_pool}).\n- `:worker_module` (module) – Defaults to `RabbitMQPoolEx.Worker.RabbitMQConnection`, which manages pool connections and channels.\n- `:size` (integer) – Number of connection processes in the pool.\n- `:channels` (integer) – Number of channels managed within the pool.\n- `:reuse_channels?` (boolean) – Defaults to `false`. Determines if channels should be reused instead of replaced after being used.\n- `:max_overflow` (integer) – Maximum number of extra workers allowed beyond the initial pool size.","ref":"RabbitMQPoolEx.html#module-configuration"},{"type":"module","title":"Example Configuration: - RabbitMQPoolEx","doc":"To use `RabbitMQPoolEx`, add the following to your application's supervision tree:\n\n```elixir\ndefmodule MyApp.Application do\n  @moduledoc false\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      {RabbitMQPoolEx.PoolSupervisor, get_pool_config()}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  defp get_pool_config do\n    [\n      rabbitmq_config: [host: \"localhost\", port: 5672],\n      connection_pools: [\n        [\n          name: {:local, :default_pool},\n          size: 5,\n          channels: 20,\n          reuse_channels?: true,\n          max_overflow: 2\n        ]\n      ]\n    ]\n  end\nend\n```","ref":"RabbitMQPoolEx.html#module-example-configuration"},{"type":"module","title":"Usage - RabbitMQPoolEx","doc":"Once configured, you can interact with RabbitMQ through the pooled connections. Here's how to publish a message:\n\n```elixir\nRabbitMQPoolEx.with_channel(:rabbitmq_pool, fn\n  {:ok, channel} ->\n    AMQP.Basic.publish(channel, \"exchange_name\", \"routing_key\", \"Hello, World!\")\n    :ok\n  {:error, reason} ->\n    IO.puts(\"Failed to acquire channel\", error: inspect(reason))\nend)\n```\n\nIn this example, `with_channel/2` checks out a channel from the pool, executes the given function, and ensures the\nchannel is returned to the pool afterward.\n\nFor more advanced usage, such as setting up consumers or handling different exchange types, refer to the detailed\ndocumentation and examples provided in the library's repository.","ref":"RabbitMQPoolEx.html#module-usage"},{"type":"module","title":"Manually retrieving a connection - RabbitMQPoolEx","doc":"To manually retrieve a RabbitMQ connection from the pool:\n\n```elixir\n{:ok, conn} = RabbitMQPoolEx.get_connection(:default_pool)\n```","ref":"RabbitMQPoolEx.html#module-manually-retrieving-a-connection"},{"type":"module","title":"Manually checking out and checking in a channel - RabbitMQPoolEx","doc":"```elixir\n{:ok, channel} = RabbitMQPoolEx.checkout_channel(worker_pid)\n\n# Perform operations...\n\nRabbitMQPoolEx.checkin_channel(worker_pid, channel)\n```\n\nIn the example above, `checkout_channel/1` retrieves a RabbitMQ channel from the connection worker,\nand `checkin_channel/2` returns it to the pool when done.","ref":"RabbitMQPoolEx.html#module-manually-checking-out-and-checking-in-a-channel"},{"type":"module","title":"Telemetry - RabbitMQPoolEx","doc":"> #### Metrics declaration {: .info}\n>\n> All metrics are defined in specific modules so you can use them like this: `RabbitMQPoolEx.Telemetry.Metrics.PoolSize.metrics()`\n> in your telemetry supervisor.\n>\n> Check the **\"Telemetry Metrics\"** section in the sidebar for a list of all available modules.\n\n`RabbitMQPoolEx` currently exposes following Telemetry events:\n\n  * `[:rabbitmq_pool_ex, :metrics, :pool_size]` - Dispatched whenever an operation increases or\n    decreases the pool size.\n\n    * Measurement: `%{count: integer}`\n    * Metadata:\n\n      ```elixir\n      %{\n        pool_id: atom,\n        channels_count: non_neg_integer,\n        reuse_channels: boolean,\n      }\n      ```\n      Check `RabbitMQPoolEx.Telemetry.Metrics.PoolSize` for more information.","ref":"RabbitMQPoolEx.html#module-telemetry"},{"type":"module","title":"License - RabbitMQPoolEx","doc":"RabbitMQPoolEx is released under the Apache 2.0 License.","ref":"RabbitMQPoolEx.html#module-license"},{"type":"function","title":"RabbitMQPoolEx.checkin_channel/2","doc":"Returns a RabbitMQ channel to its corresponding connection worker.","ref":"RabbitMQPoolEx.html#checkin_channel/2"},{"type":"function","title":"Parameters - RabbitMQPoolEx.checkin_channel/2","doc":"- `conn_worker`: PID of the connection worker.\n  - `channel`: The RabbitMQ channel to be returned.","ref":"RabbitMQPoolEx.html#checkin_channel/2-parameters"},{"type":"function","title":"RabbitMQPoolEx.checkout_channel/1","doc":"Retrieves a RabbitMQ channel from the specified connection worker.","ref":"RabbitMQPoolEx.html#checkout_channel/1"},{"type":"function","title":"Parameters - RabbitMQPoolEx.checkout_channel/1","doc":"- `conn_worker`: PID of the connection worker.","ref":"RabbitMQPoolEx.html#checkout_channel/1-parameters"},{"type":"type","title":"RabbitMQPoolEx.client_function/0","doc":"The function to be used with `with_channel/2`","ref":"RabbitMQPoolEx.html#t:client_function/0"},{"type":"function","title":"RabbitMQPoolEx.get_connection/1","doc":"Retrieves a RabbitMQ connection from a connection worker within the pool.","ref":"RabbitMQPoolEx.html#get_connection/1"},{"type":"function","title":"Parameters - RabbitMQPoolEx.get_connection/1","doc":"- `pool_id`: Atom representing the pool identifier.","ref":"RabbitMQPoolEx.html#get_connection/1-parameters"},{"type":"function","title":"RabbitMQPoolEx.get_connection_worker/1","doc":"Retrieves a connection worker from the pool.\n\nThis function uses the pool for load distribution and does not isolate access to individual workers.\nThe pool configuration strategy is FIFO.","ref":"RabbitMQPoolEx.html#get_connection_worker/1"},{"type":"function","title":"Parameters - RabbitMQPoolEx.get_connection_worker/1","doc":"- `pool_id:` Atom representing the pool identifier.","ref":"RabbitMQPoolEx.html#get_connection_worker/1-parameters"},{"type":"function","title":"RabbitMQPoolEx.with_channel/2","doc":"Executes a given function within the context of a RabbitMQ channel.\n\nThis function:\n  1. Retrieves the reference (pid) of connection worker from the pool and\n    immediately put it back into the pool, so any other concurrent client\n    can have access to it.\n  2. Obtains a RabbitMQ channel from that worker.\n  3. Executes the provided function using the channel.\n  4. Returns the channel to the worker's pool.","ref":"RabbitMQPoolEx.html#with_channel/2"},{"type":"function","title":"Parameters - RabbitMQPoolEx.with_channel/2","doc":"- `pool_id`: Atom representing the pool identifier.\n  - `fun`: Function to be executed within the channel's context.","ref":"RabbitMQPoolEx.html#with_channel/2-parameters"},{"type":"module","title":"RabbitMQPoolEx.Adapters.RabbitMQ","doc":"The default adapter for interacting with RabbitMQ using the `AMQP` library.\n\nThis module implements the `RabbitMQPoolEx.Ports.RabbitMQ` behavior, providing concrete implementations\nthat delegate directly to the `AMQP.Connection` and `AMQP.Channel` modules.","ref":"RabbitMQPoolEx.Adapters.RabbitMQ.html"},{"type":"function","title":"RabbitMQPoolEx.Adapters.RabbitMQ.close_channel/1","doc":"","ref":"RabbitMQPoolEx.Adapters.RabbitMQ.html#close_channel/1"},{"type":"function","title":"RabbitMQPoolEx.Adapters.RabbitMQ.close_connection/1","doc":"","ref":"RabbitMQPoolEx.Adapters.RabbitMQ.html#close_connection/1"},{"type":"function","title":"RabbitMQPoolEx.Adapters.RabbitMQ.open_channel/1","doc":"","ref":"RabbitMQPoolEx.Adapters.RabbitMQ.html#open_channel/1"},{"type":"function","title":"RabbitMQPoolEx.Adapters.RabbitMQ.open_connection/1","doc":"","ref":"RabbitMQPoolEx.Adapters.RabbitMQ.html#open_connection/1"},{"type":"behaviour","title":"RabbitMQPoolEx.Ports.RabbitMQ","doc":"A behavior module defining the contract for interacting with RabbitMQ connections and channels.\n\nThis module acts as a port, abstracting the interaction with the RabbitMQ server using the `AMQP` library.\nIt provides callbacks that define the expected behavior for opening and closing both connections and channels.\n\nImplementations of this module should adhere to these callbacks to ensure consistent communication with RabbitMQ.","ref":"RabbitMQPoolEx.Ports.RabbitMQ.html"},{"type":"behaviour","title":"Callbacks - RabbitMQPoolEx.Ports.RabbitMQ","doc":"- `c:open_connection/1`:\n  Opens a connection to the RabbitMQ server using either a keyword list or a connection string.\n  Returns `{:ok, Connection.t()}` on success or `{:error, reason}` on failure.\n\n- `c:close_connection/1`:\n  Closes an active RabbitMQ connection.\n  Returns `:ok` on success or `{:error, reason}` on failure.\n\n- `c:open_channel/1`:\n  Opens a new channel within an existing RabbitMQ connection.\n  Returns `{:ok, Channel.t()}` on success or `{:error, reason}` on failure.\n\n- `c:close_channel/1`:\n  Closes an active RabbitMQ channel.\n  Returns `:ok` on success or `{:error, AMQP.Basic.error()}` on failure.","ref":"RabbitMQPoolEx.Ports.RabbitMQ.html#module-callbacks"},{"type":"behaviour","title":"Example - RabbitMQPoolEx.Ports.RabbitMQ","doc":"To implement this behavior in a module:\n```elixir\n  defmodule MyApp.RabbitMQ do\n    @behaviour RabbitMQPoolEx.Ports.RabbitMQ\n\n    def open_connection(config) do\n      AMQP.Connection.open(config)\n    end\n\n    def close_connection(connection) do\n      AMQP.Connection.close(connection)\n    end\n\n    def open_channel(connection) do\n      AMQP.Channel.open(connection)\n    end\n\n    def close_channel(channel) do\n      AMQP.Channel.close(channel)\n    end\n  end\n```\n\nThis design allows for easy mocking and testing by providing alternative implementations as needed.","ref":"RabbitMQPoolEx.Ports.RabbitMQ.html#module-example"},{"type":"callback","title":"RabbitMQPoolEx.Ports.RabbitMQ.close_channel/1","doc":"","ref":"RabbitMQPoolEx.Ports.RabbitMQ.html#c:close_channel/1"},{"type":"callback","title":"RabbitMQPoolEx.Ports.RabbitMQ.close_connection/1","doc":"","ref":"RabbitMQPoolEx.Ports.RabbitMQ.html#c:close_connection/1"},{"type":"callback","title":"RabbitMQPoolEx.Ports.RabbitMQ.open_channel/1","doc":"","ref":"RabbitMQPoolEx.Ports.RabbitMQ.html#c:open_channel/1"},{"type":"callback","title":"RabbitMQPoolEx.Ports.RabbitMQ.open_connection/1","doc":"","ref":"RabbitMQPoolEx.Ports.RabbitMQ.html#c:open_connection/1"},{"type":"module","title":"RabbitMQPoolEx.Worker.RabbitMQConnection","doc":"Worker module for managing RabbitMQ connections and channel pooling using `poolboy`.\n\nThis module establishes and maintains a connection to RabbitMQ, manages a pool of channels,\nand handles reconnections and channel restarts in case of failures. It ensures that channels\nare efficiently reused and that clients can safely checkout and checkin channels.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html"},{"type":"module","title":"Configuration Options - RabbitMQPoolEx.Worker.RabbitMQConnection","doc":"* `:reuse_channels?` - When set to `true`, channels are reused when checked back into the pool. Defaults to `false`.\n  * `:channels` - Number of channels to create in the pool. Defaults to 10.\n  * `:reconnect_interval` - Interval in milliseconds to wait before attempting to reconnect. Defaults to 1000ms.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#module-configuration-options"},{"type":"function","title":"RabbitMQPoolEx.Worker.RabbitMQConnection.checkin_channel/2","doc":"Checks a channel back into the pool.\n\nIt takes care of reintegrating the channel into the state,\nensuring the channel is properly monitored, and that the connection status is properly managed.\n\nThe function behaves differently based on whether the channel reuse feature is enabled (`reuse_channels?: true`) in the worker state.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#checkin_channel/2"},{"type":"function","title":"Parameters: - RabbitMQPoolEx.Worker.RabbitMQConnection.checkin_channel/2","doc":"- `pid` - The PID of the worker process.\n  - `channel` - The channel to check back in.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#checkin_channel/2-parameters"},{"type":"function","title":"Behavior: - RabbitMQPoolEx.Worker.RabbitMQConnection.checkin_channel/2","doc":"- When `reuse_channels?: true`:\n  - If the channel's `pid` already exists in the pool (i.e., in the list of channels), it will simply remove the monitor for that `pid` and return the updated state.\n  - If the `pid` is not already in the pool, it will add the channel to the list of channels and remove the monitor for that `pid` and return the updated state\n\n- When `reuse_channels?` is not enabled:\n  - If the channel's `pid` is found in either the list of channels or monitors, it will attempt to remove the channel from the pool and handle potential errors in case the channel has been closed.\n  - If the channel's connection is still valid, it will replace the channel with a new one (close the current channel and open a new one).\n  - If the connection is closed (`{:error, :closing}`), the state will simply be updated without re-adding the channel.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#checkin_channel/2-behavior"},{"type":"function","title":"RabbitMQPoolEx.Worker.RabbitMQConnection.checkout_channel/1","doc":"Checks out a channel from the pool.\n\nIt removes the channel from the available channels list\nand monitors the caller process (`from_pid`) to ensure proper cleanup in case of failure.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#checkout_channel/1"},{"type":"function","title":"Parameters: - RabbitMQPoolEx.Worker.RabbitMQConnection.checkout_channel/1","doc":"- `from_pid` (`pid`): The process identifier of the client (caller) that is requesting the channel.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#checkout_channel/1-parameters"},{"type":"function","title":"Behavior: - RabbitMQPoolEx.Worker.RabbitMQConnection.checkout_channel/1","doc":"- If there are available channels in the pool (`channels` list is non-empty), the function will:\n  - Select the first channel (`channel`), remove it from the list of available channels (`channels: rest`), and add the caller process (`from_pid`) as a monitor for the channel.\n  - The monitor reference is added to the `monitors` map, linking the `pid` of the channel to the new monitor reference.\n  - The function returns a tuple containing the checked-out `channel` and the updated `state` with the modified list of channels and the updated monitor map.\n\n- If no channels are available, the function will raise an error or behave as defined in other parts of the system, though this behavior is not detailed here.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#checkout_channel/1-behavior"},{"type":"function","title":"RabbitMQPoolEx.Worker.RabbitMQConnection.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#child_spec/1"},{"type":"function","title":"RabbitMQPoolEx.Worker.RabbitMQConnection.create_channel/1","doc":"Creates a new channel outside of the pool.\n\nThis allows clients to manually create channels without automatic management or pooling.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#create_channel/1"},{"type":"function","title":"Parameters - RabbitMQPoolEx.Worker.RabbitMQConnection.create_channel/1","doc":"- `pid` - The PID of the worker process.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#create_channel/1-parameters"},{"type":"function","title":"RabbitMQPoolEx.Worker.RabbitMQConnection.get_connection/1","doc":"Retrieves the current RabbitMQ connection.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#get_connection/1"},{"type":"function","title":"Parameters - RabbitMQPoolEx.Worker.RabbitMQConnection.get_connection/1","doc":"- `pid` - The PID of the worker process.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#get_connection/1-parameters"},{"type":"function","title":"RabbitMQPoolEx.Worker.RabbitMQConnection.init/1","doc":"Initializes the worker process by trapping exits, ensuring that all linked connections\nand multiplexed channels can be restarted by the worker.\n\nThis function also triggers an asynchronous connection attempt, ensuring that any future\ncalls will wait until the connection is successfully established.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#init/1"},{"type":"function","title":"Parameters - RabbitMQPoolEx.Worker.RabbitMQConnection.init/1","doc":"- `amqp_config` (keyword or string): RabbitMQ configuration settings used to establish the connection.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#init/1-parameters"},{"type":"function","title":"RabbitMQPoolEx.Worker.RabbitMQConnection.start_link/1","doc":"Starts the RabbitMQ connection worker.","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#start_link/1"},{"type":"function","title":"Parameters - RabbitMQPoolEx.Worker.RabbitMQConnection.start_link/1","doc":"- `config` - Configuration options (keyword list or string).","ref":"RabbitMQPoolEx.Worker.RabbitMQConnection.html#start_link/1-parameters"},{"type":"module","title":"RabbitMQPoolEx.Worker.State","doc":"Represents the state of the RabbitMQ connection worker.","ref":"RabbitMQPoolEx.Worker.State.html"},{"type":"module","title":"Fields - RabbitMQPoolEx.Worker.State","doc":"* `:connection` - The active RabbitMQ connection.\n  * `:channels` - List of channels available in the pool.\n  * `:monitors` - Map of monitored processes holding channels.\n  * `:config` - Configuration options provided when starting the worker.\n  * `:reuse_channels?` - Boolean indicating if channels are reused.","ref":"RabbitMQPoolEx.Worker.State.html#module-fields"},{"type":"type","title":"RabbitMQPoolEx.Worker.State.config/0","doc":"","ref":"RabbitMQPoolEx.Worker.State.html#t:config/0"},{"type":"type","title":"RabbitMQPoolEx.Worker.State.t/0","doc":"","ref":"RabbitMQPoolEx.Worker.State.html#t:t/0"},{"type":"module","title":"RabbitMQPoolEx.Telemetry.Metrics.PoolSize","doc":"Telemetry metrics for monitoring the pool size of RabbitMQ connections.\n\nThis module defines telemetry metrics related to the size of the connection pool in `RabbitMQPoolEx`.\nIt provides a metric for tracking the last known pool size and emits telemetry events when the pool size changes.","ref":"RabbitMQPoolEx.Telemetry.Metrics.PoolSize.html"},{"type":"function","title":"RabbitMQPoolEx.Telemetry.Metrics.PoolSize.metrics/0","doc":"Returns a list of telemetry metrics for monitoring the RabbitMQ connection pool size.\n\nThis function defines the following metric:\n\n- `rabbitmq_pool_ex.metrics.pool_size.count`: Tracks the latest pool size value.","ref":"RabbitMQPoolEx.Telemetry.Metrics.PoolSize.html#metrics/0"},{"type":"function","title":"Metadata - RabbitMQPoolEx.Telemetry.Metrics.PoolSize.metrics/0","doc":"The event includes the following metadata as tags:\n\n- `:pool_id` - The identifier of the connection pool.\n- `:channels_count` - The number of active channels in the connection.\n- `:reuse_channels` - Whether channels are being reused.","ref":"RabbitMQPoolEx.Telemetry.Metrics.PoolSize.html#metrics/0-metadata"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [1.1.0]","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Telemetry metric for pool_size\n\n## [1.0.0]","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Downgrade AMQP version to 3 to ease integration with old apps","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Use the correct supervisor in the docs example\n\n## [1.0.0-alpha.1]","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- RabbitMQ connection and channel pools","ref":"changelog.html#added"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}